<?php
/**
 *
 * Quote depth fix. An extension for the phpBB Forum Software package.
 *
 * @copyright (c) 2019, Pazh, https://phpbbgurunet.ru
 * @license GNU General Public License, version 2 (GPL-2.0)
 *
 */

namespace pazh\quotedepth\event;

/**
 * @ignore
 */
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Quote depth fix Event listener.
 */
class listener implements EventSubscriberInterface
{
	/** @var \phpbb\auth\auth */
	protected $auth;
	
	/** @var \phpbb\config\config */
	protected $config;

	/** @var ContainerInterface */
	protected $phpbb_container;
	
	protected $enable_remove;

	/**
	 * listener constructor.
	 *
	 * @param \phpbb\auth\auth                  $auth
	 * @param \phpbb\config\config              $config
	 * @param ContainerInterface 				$phpbb_container
	 */
	public function __construct(\phpbb\auth\auth $auth, \phpbb\config\config $config, ContainerInterface $phpbb_container)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->phpbb_container = $phpbb_container;
		$this->enable_remove = false;
	}
	
	static public function getSubscribedEvents()
	{
		return array(
			'core.posting_modify_template_vars' => 'posting_modify_template_vars',
			'core.viewtopic_modify_post_action_conditions' => 'viewtopic_modify_post_action_conditions',
			'core.decode_message_before'		=> 'decode_message_before',
		);
	}

	// For Full Editor
	public function posting_modify_template_vars($event)
	{
		if ($event['mode'] == 'reply' || $event['mode'] == 'quote')
		{
			$this->enable_remove = true;
		}
	}
	
	// For QuickReply Editor
	public function viewtopic_modify_post_action_conditions($event)
	{
		if ($this->config['qr_full_quote'] && $this->auth->acl_get('f_reply', $event['topic_data']['forum_id']))
		{
			$this->enable_remove = true;
		}
	}
	
	public function decode_message_before($event)
	{
		if ($this->enable_remove && $this->config['max_quote_depth'] > 0)
		{
			$message_text = $event['message_text'];
			$bbcode_uid = $event['bbcode_uid'];
			$this->remove_nested_quotes($message_text, $bbcode_uid, $this->config['max_quote_depth']-1);
			$event['message_text'] = $message_text;
		}	
	}
	
	/**
	* Remove nested quotes at given depth in current parsed message
	*
	* @param  integer $max_depth Depth limit
	* @return null
	*/
	private function remove_nested_quotes(&$message, $bbcode_uid, $max_depth)
	{
		if (preg_match('#^<[rt][ >]#', $message))
		{
			$message = $this->phpbb_container->get('text_formatter.utils')->remove_bbcode(
				$message,
				'quote',
				$max_depth
			);

			return;
		}

		// Capture all [quote] and [/quote] tags
		preg_match_all('(\\[/?quote(?:=&quot;(.*?)&quot;)?:' . $bbcode_uid . '\\])', $message, $matches, PREG_OFFSET_CAPTURE);

		// Iterate over the quote tags to mark the ranges that must be removed
		$depth = 0;
		$ranges = array();
		$start_pos = 0;
		foreach ($matches[0] as $match)
		{
			if ($match[0][1] === '/')
			{
				--$depth;
				if ($depth == $max_depth)
				{
					$end_pos = $match[1] + strlen($match[0]);
					$length = $end_pos - $start_pos;
					$ranges[] = array($start_pos, $length);
				}
			}
			else
			{
				++$depth;
				if ($depth == $max_depth + 1)
				{
					$start_pos = $match[1];
				}
			}
		}

		foreach (array_reverse($ranges) as $range)
		{
			list($start_pos, $length) = $range;
			$message = substr_replace($message, '', $start_pos, $length);
		}
	}
}
